﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using NHibernate.Cfg;
using NHibernate.Dialect;
using NHibernate.Driver;
using System.Reflection;
using NHibernate;
//Located at: https://data.itreetools.org/sql
using LocationSpecies.Domain; 

namespace WeatherPrep
{
    class CreateOutput
    {
        //JK 20211101: Apparently these fields are never used
        //string locDB;
        //string nationID;
        //string priPartID;
        //string secPartID;
        //string terPartID;

        double LeafAreaIndexMax;
        double Evergreen_pct;
        VEGTYPE VegetationType;
        int startYr;
        int endYr;
        double Height_WeatherStationWindSensor_m;
        double Height_TreeCanopyTop_m;

        string MeteorologicalFile_string;
        string PrecipitationFile_string;
        string LeafAreaIndex_dataBase;
        string WeatherVariables_dBase;
        string model_type;

        // Variables sent by ReadInputXML.cs: 
        //MeteorologicalDataInputFile_string, PrecipitationDataInputFile_string, LocationData_Object, LeafAreaIndex_dataBase, VegetationType_class, startYear, endYear, WeatherVariables_dataBase

        public CreateOutput(double LeafAreaIndex_max, double Evergreen_percent, string LeafAreaIndex_dataBase_string, int startYear, int endYear,
            VEGTYPE VegetationType_class, string MeteorologicalDataInputFile_string, string PrecipitationDataInputFile_string, string WeatherVariables_dataBase, string ModelType_string,
            double Height_WeatherStationWindSensor_t_m, double Height_TreeCanopyTop_t_m)
        {
            LeafAreaIndexMax = LeafAreaIndex_max;
            Evergreen_pct = Evergreen_percent;
            VegetationType = VegetationType_class;
            Height_WeatherStationWindSensor_m = Height_WeatherStationWindSensor_t_m;
            Height_TreeCanopyTop_m = Height_TreeCanopyTop_t_m;
            startYr = startYear;
            endYr = endYear;
            MeteorologicalFile_string = MeteorologicalDataInputFile_string;
            PrecipitationFile_string = PrecipitationDataInputFile_string;
            LeafAreaIndex_dataBase = LeafAreaIndex_dataBase_string;
            WeatherVariables_dBase = WeatherVariables_dataBase;
            model_type = ModelType_string;
        }

        // JK 20211029: Function now works with CSV files only
        public void CreateOutputFiles(ISession session, Location location)
        {
            try
            {
                LocationData LocationData_Object = new LocationData(session);
                ForestData forData;
                LocationData_Object.ReadLocationData(location);
                forData = new ForestData(0, LeafAreaIndexMax, Evergreen_pct, 0);

                Console.WriteLine("\nQuerying server to determine leaf on and off dates and compute meteorological outputs.");
                //call LeafAreaIndex.ProcessLAI to compute terms for LAI database
                LeafAreaIndex.ProcessLAI(forData.MaxLAI, forData.PctEvGrnCov,
                                        LocationData_Object, startYr, endYr, LeafAreaIndex_dataBase);
                //Optional console output for debbugging
                //Console.WriteLine("Leaf area index timeseries DB constructed for use in evapotranspiration calculations.");

                //call ProcessSurfaceWeatherData to compute terms for weather database
                //Note: Variables sent to SurfaceWeather.cs ...
                //Note: ...  SurfaceWeatherDataFile, RainDataExcel, LocationData, LAIDatabase, vegType, StartYear, EndYear, WeatherDatabase, Height_WindSensor_m, Height_Tree_m
                SurfaceWeather.ProcessSurfaceWeatherData(MeteorologicalFile_string, PrecipitationFile_string, LocationData_Object, LeafAreaIndex_dataBase,
                                                        VegetationType, startYr, endYr, WeatherVariables_dBase,
                                                        Height_WeatherStationWindSensor_m, Height_TreeCanopyTop_m);
                //Optional console output for debbugging
                //Console.WriteLine("Meteorological timeseries DB constructed to generate output data files.\n");

                //Console output to manage expectations for upcoming program activity
                Console.WriteLine("Generating Weather, Evaporation, and Radiation data files ...");

                //assign names to output files, Weather.csv, Evaporation.csv, Radiation.csv
                string weatherFile = Path.GetDirectoryName(WeatherVariables_dBase) + "\\Weather.csv";
                string evFile = Path.GetDirectoryName(WeatherVariables_dBase) + "\\Evaporation.csv";
                string srFile = Path.GetDirectoryName(WeatherVariables_dBase) + "\\Radiation.csv";

                //Write output to user
                Console.WriteLine("The following output files were written:");
                //call CreateWeatherDatCSV to create Weather.csv output
                CreateWeatherDatCSV(WeatherVariables_dBase, weatherFile);
                Console.WriteLine(weatherFile);

                //call CreateEvapDatCSV to create Evaporation.csv output
                CreateEvapDatCSV(WeatherVariables_dBase, evFile);
                Console.WriteLine(evFile);

                //call CreateSolarRadDatCSV to create Radiation.csv output
                CreateSolarRadDatCSV(WeatherVariables_dBase, srFile);
                Console.WriteLine(srFile);

                //Note: LAIDatabase & WeatherDatabase are written to the default dirPath, defined above
                Console.WriteLine("Databases used to generate output, LAI.csv and Meteo.csv, were also provided.");

                //Write output to user
                Console.WriteLine("\nSimulation completed. Check above output files for results.");
                Console.WriteLine("Thank you for using i-Tree tools to improve the world!");
            }
            catch (Exception)
            {
                return;
            }
        }

        //CreateWeatherDatCSV function uses WeatherVariables_dBase from CSV file and weatherFile strings
        private void CreateWeatherDatCSV(string WeatherVariables_dBase, string weatherFile)
        {
            try
            {
                //Dimensionalize list variables to contain weather data
                List<DateTime> dtList = new List<DateTime>();
                List<double> Tair_F = new List<double>();
                List<double> Tdew_F = new List<double>();
                List<double> NetRadWm2 = new List<double>();
                List<double> WindSpeedms = new List<double>();
                List<double> Precipmh = new List<double>();
                List<double> Snowmh = new List<double>();
                List<double> WindDir_deg = new List<double>();
                List<double> AtmPressure_kPa = new List<double>();

                using (StreamReader WeatherVariables_StreamReader = new StreamReader(WeatherVariables_dBase))
                {
                    string Line_MeteorologicalVariables;
                    //read the header Line_MeteorologicalVariables
                    Line_MeteorologicalVariables = WeatherVariables_StreamReader.ReadLine();

                    //Loop through WeatherVariables_dBase and append weather data into new list variables
                    while ((Line_MeteorologicalVariables = WeatherVariables_StreamReader.ReadLine()) != null)
                    {
                        dtList.Add(DateTime.Parse(Line_MeteorologicalVariables.Split(',')[0]));
                        Tair_F.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[50]));
                        Tdew_F.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[53]));
                        NetRadWm2.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[41]));
                        WindSpeedms.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[57]));
                        Precipmh.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[43]));
                        Snowmh.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[47]));
                        WindDir_deg.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[55]));
                        AtmPressure_kPa.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[13]));
                    }
                }

                //Create sw as CSV file of weather data output using StreamWriter function
                using (StreamWriter sw = File.CreateText(weatherFile))
                {
                    //Write header to sw CSV file of weather data output
                    //Hydro model defined in WeatherPrepConfig.xml, for use w/ i-Tree Hydro input and contains 9 columns of variables
                    if (model_type == "Hydro")
                    {
                        sw.WriteLine("YYYYMMDD,HH:MM:SS,Temperature_Air_F(F),Temperature_DewPoint_F(F),Radiation_Net_Wpm2(W/m^2),Wind_Speed_mps(m/s),Pressure_Atmosphere_kPa(kPa),Precipitation_Rate_mph(m/h),Snow_DepthOnGround_m(m)");
                    }

                    //Energy model defined in WeatherPrepConfig.xml, for use w/ i-Tree Energy input and contains 10 columns of variables
                    if (model_type == "Energy")
                    {
                        sw.WriteLine("YYYYMMDD,HH:MM:SS,Tair_F(F),Tdew_F(F),NetRad_Wpm2(W/m^2),WndSpd_mps(m/s),WindDir_deg(deg),AtmPres_kPa(kPa),Precip_mph(m/h),SnowDepth_m(m)");
                    }
                    //HydroEvap model defined in WeatherPrepConfig.xml, for use w/ i-Tree Hydro input and contains 8 columns of variables
                    if (model_type == "Hydro8")
                    {
                        sw.WriteLine("YYYYMMDD,HH:MM:SS,Tair_F(F),Tdew_F(F),NetRad_Wpm2(W/m^2),WndSpd_mps(m/s),Precip_mph(m/h),SnowDepth_m(m)");
                    }

                    //Loop through index i from 0 to less than total items in variable list
                    for (int i = 0; i < dtList.Count; i++)
                    {
                        //HydroEvap model defined in WeatherPrepConfig.xml, for use w/ i-Tree Hydro w/ evaporation input and contains 9 columns of variables
                        if (model_type == "Hydro")
                        {
                            //Write data to sw CSV file, formatting to 8 digits precision
                            //Note: yyyyMMdd is format required by C# to write 8 digit date
                            sw.Write(dtList[i].Date.ToString("yyyyMMdd") + "," +
                            (dtList[i].Hour.ToString("00") + ":00:00") + "," +
                            First(Tair_F[i].ToString("0.00000000"), 11) + "," +
                            First(Tdew_F[i].ToString("0.00000000"), 11) + "," +
                            First(NetRadWm2[i].ToString("0.00000000"), 11) + "," +
                            First(WindSpeedms[i].ToString("0.00000000"), 11) + "," +
                            First(AtmPressure_kPa[i].ToString("0.00000000"), 11) + "," +
                            First(Precipmh[i].ToString("0.00000000"), 11) + "," +
                            First(Snowmh[i].ToString("0.00000000"), 11) + "\r\n");
                        }
                        //Energy model defined in WeatherPrepConfig.xml, for use w/ i-Tree Energy input and contains 10 columns of variables
                        if (model_type == "Energy")
                        {
                            //Write data to sw CSV file, formatting to 8 digits precision
                            //Note: yyyyMMdd is format required by C# to write 8 digit date
                            sw.Write(dtList[i].Date.ToString("yyyyMMdd") + "," +
                            (dtList[i].Hour.ToString("00") + ":00:00") + "," +
                            First(Tair_F[i].ToString("0.00000000"), 11) + "," +
                            First(Tdew_F[i].ToString("0.00000000"), 11) + "," +
                            First(NetRadWm2[i].ToString("0.00000000"), 11) + "," +
                            First(WindSpeedms[i].ToString("0.00000000"), 11) + "," +
                            First(WindDir_deg[i].ToString("0.00000000"), 11) + "," +
                            First(AtmPressure_kPa[i].ToString("0.00000000"), 11) + "," +
                            First(Precipmh[i].ToString("0.00000000"), 11) + "," +
                            First(Snowmh[i].ToString("0.00000000"), 11) + "\r\n");
                        }
                        //Hydro model defined in WeatherPrepConfig.xml, for use w/ i-Tree Hydro input and contains 8 columns of variables
                        if (model_type == "Hydro8")
                        {
                            //Write data to sw CSV file, formatting to 8 digits precision
                            //Note: yyyyMMdd is format required by C# to write 8 digit date
                            sw.Write(dtList[i].Date.ToString("yyyyMMdd") + "," +
                            (dtList[i].Hour.ToString("00") + ":00:00") + "," +
                            First(Tair_F[i].ToString("0.00000000"), 11) + "," +
                            First(Tdew_F[i].ToString("0.00000000"), 11) + "," +
                            First(NetRadWm2[i].ToString("0.00000000"), 11) + "," +
                            First(WindSpeedms[i].ToString("0.00000000"), 11) + "," +
                            First(Precipmh[i].ToString("0.00000000"), 11) + "," +
                            First(Snowmh[i].ToString("0.00000000"), 11) + "\r\n");
                        }
                    }
                    //Close sw CSV file
                    sw.Close();
                }
            }
            catch (Exception)
            {
                throw;
            }
        }
        private void CreateEvapDatCSV(string WeatherVariables_dBase, string evFile)
        {
            try
            {
                List<DateTime> dtList = new List<DateTime>();
                List<double> pET = new List<double>(); //pET.dat PtTrMh
                List<double> pE = new List<double>(); // pE.dat PeGrMh
                List<double> pETree = new List<double>(); //pEvTree -> pETree.dat PeTrMh
                List<double> PESnow = new List<double>();
                List<double> PeTreeSnow = new List<double>();

                using (StreamReader WeatherVariables_StreamReader = new StreamReader(WeatherVariables_dBase))
                {
                    string Line_MeteorologicalVariables;
                    //read the header Line_MeteorologicalVariables
                    WeatherVariables_StreamReader.ReadLine();

                    //Loop through WeatherVariables_dBase and append weather data into new list variables
                    while ((Line_MeteorologicalVariables = WeatherVariables_StreamReader.ReadLine()) != null)
                    {
                        dtList.Add(DateTime.Parse(Line_MeteorologicalVariables.Split(',')[0]));
                        pET.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[19]));
                        pE.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[16]));
                        pETree.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[15]));
                        PESnow.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[18]));
                        PeTreeSnow.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[17]));
                    }
                }
                using (StreamWriter sw = File.CreateText(evFile))
                {
                    //Write header line for Evaporation.csv
                    sw.WriteLine("YYYYMMDD,HH:MM:SS,PotentialEvapotranspiration_TreeCover_mph(m/h),PotentialEvaporation_WaterOnGround_mph(m/h),PotentialEvaporation_WaterOnTree_mph(m/h),PotentialSublimation_SnowOnGround_mph(m/h),PotentialSublimation_SnowOnTree_mph(m/h)");

                    for (int i = 0; i < dtList.Count; i++)
                    {
                        //Note: yyyyMMdd is format required by C# to write 8 digit date
                        sw.Write(dtList[i].Date.ToString("yyyyMMdd") + "," +
                            (dtList[i].Hour.ToString("00") + ":00:00") + "," +
                            First(pET[i].ToString("0.00000000"), 11) + "," +
                            First(pE[i].ToString("0.00000000"), 11) + "," +
                            First(pETree[i].ToString("0.00000000"), 11) + "," +
                            First(PESnow[i].ToString("0.00000000"), 11) + "," +
                            First(PeTreeSnow[i].ToString("0.00000000"), 11) + "\r\n");
                    }
                    sw.Close();
                }
            }
            catch (Exception)
            {
                throw;
            }
        }
        private void CreateSolarRadDatCSV(string WeatherVariables_dBase, string srFile)
        {
            try
            {
                List<DateTime> dtList = new List<DateTime>();
                List<double> DirSwWm2 = new List<double>();
                List<double> DiffSwWm2 = new List<double>();
                List<double> DownLwWm2 = new List<double>();
                List<double> UpLwWm2 = new List<double>();
                List<double> LatRad = new List<double>();
                List<double> HrAngleRad = new List<double>();
                List<double> DecAngleRad = new List<double>();

                using (StreamReader WeatherVariables_StreamReader = new StreamReader(WeatherVariables_dBase))
                {
                    string Line_MeteorologicalVariables;
                    //read the header Line_MeteorologicalVariables
                    WeatherVariables_StreamReader.ReadLine();

                    //Loop through WeatherVariables_dBase and append weather data into new list variables
                    while ((Line_MeteorologicalVariables = WeatherVariables_StreamReader.ReadLine()) != null)
                    {
                        dtList.Add(DateTime.Parse(Line_MeteorologicalVariables.Split(',')[0]));
                        DirSwWm2.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[36]));
                        DiffSwWm2.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[37]));
                        DownLwWm2.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[1]));
                        UpLwWm2.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[2]));
                        LatRad.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[3]));
                        HrAngleRad.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[4]));
                        DecAngleRad.Add(Double.Parse(Line_MeteorologicalVariables.Split(',')[5]));
                    }
                }
                using (StreamWriter sw = File.CreateText(srFile))
                {
                    sw.WriteLine("YYYYMMDD,HH:MM:SS,Radiation_Shortwave_Direct_Wpm2(W/m^2),Radiation_Shortwave_Diffuse_Wpm2(W/m^2),Radiation_Longwave_Downwelling_Wpm2(W/m^2),Radiation_Longwave_Upwelling_Wpm2(W/m^2),Latitude_rad(Radian),HourAngle_rad(Radian),DeclinationAngle_rad(Radian)");

                    for (int i = 0; i < dtList.Count; i++)
                    {
                        //Note: yyyyMMdd is format required by C# to write 8 digit date
                        sw.Write(dtList[i].Date.ToString("yyyyMMdd") + "," +
                            (dtList[i].Hour.ToString("00") + ":00:00") + "," +
                             First(DirSwWm2[i].ToString("0.00000"), 7) + "," +
                             First(DiffSwWm2[i].ToString("0.00000"), 7) + "," +
                             First(DownLwWm2[i].ToString("0.00000"), 7) + "," +
                             First(UpLwWm2[i].ToString("0.00000"), 7) + "," +
                             First(LatRad[i].ToString("0.00000"), 7) + "," +
                             First(HrAngleRad[i].ToString("0.00000"), 7) + "," +
                             First(DecAngleRad[i].ToString("0.00000"), 7) + "\r\n");
                    }
                    sw.Close();
                }
            }
            catch (Exception)
            {
                throw;
            }
        }
        private String First(string p, int p_2)
        {
            return p.Substring(0, Math.Min(p.Length, p_2));
        }
    }
}

