﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.IO;
using SqlWeb;
using SqlWeb.Data;
using NHibernate;
using LocationSpecies.Domain;
using NHibernate.Cfg;
using NHibernate.Dialect;
using NHibernate.Driver;
using System.Reflection;
using LocationSpecies;
using NHibernate.Criterion;
using NHibernate.Linq;
using NHibernate.SqlCommand;

namespace WeatherPrep
{

    class ReadInputXML
    {
        //Defining string variables LocationTypes_Vector that are read in from WeatherPrepConfig.xml
        //Note: WeatherPrepConfig.xml is read below in XmlReader.Read() and InputDataXml.Name.ToString()
        public static VEGTYPE vegType;
        public static int StartYear, EndYear;
        public static String Model;
        public static String Nation;
        public static String State;
        public static String County;
        public static String Place;
        public static double Height_Tree_m;
        public static double Height_WindSensor_m;
        public static String MaximumLAI;
        public static String EvergreenPercent;
        public static String VegetationType;
        public static String Type;

        //public static String LAIDatabase;
        public static String SurfaceWeatherDataFile;
        public static String PrecipitationDataCsv;
        //public static String WeatherDatabase;

        //Location LookupLocation is called from Main
        public static Location LookupLocation(ISession session)
        {
            //ITransaction created as a new NHibernate_Session
            using (ITransaction tx = session.BeginTransaction())
            {
                //LocationNames_Vector defined to contain four attributes of location
                string[] LocationNames_Vector = { Place, County, State, Nation };
                //LocationTypes_Vector defined to contain two attributes of location type that will be excluded
                var LocationTypes_Vector = new[] { LocationType.CountySubdivision, LocationType.CensusSubdivision };

                //Query_NHibernate defined with NHibernate QueryOver for session Location
                //  Where Type != LocationType CountySubdivision and CensusSubdivision
                var Query_NHibernate = session.QueryOver<Location>()
                    .Where(Query_Data01 => !Query_Data01.Type.IsIn(LocationTypes_Vector));

                //For Loop i_counter through all LocationNames_Vector members
                for (int i_counter = 0; i_counter < LocationNames_Vector.Length; i_counter++)
                {
                    //LocationName defined from LocationNames_Vector
                    var LocationName = LocationNames_Vector[i_counter];

                    //If i_counter greater than 0 then enter
                    if (!String.IsNullOrEmpty(LocationName))
                    {
                        if (i_counter > 0)
                        {
                            //Query_NHibernate constructed to contain JoinType information
                            Query_NHibernate = Query_NHibernate
                                .JoinQueryOver<LocationRelation>(Query_Data01 => Query_Data01.Relations, JoinType.InnerJoin)
                                .JoinQueryOver(Query_Data02 => Query_Data02.Parent, JoinType.InnerJoin);
                        }

                        //Query_NHibernate redefined with Where command using two criteria: Name ~ LocationName
                        //Note: Where statement above has Type != CountySubdivision != CensusSubdivision
                        Query_NHibernate = Query_NHibernate.Where(Query_Data01 => Query_Data01.Name.IsLike(LocationName));
                    }
                }
                //return Query_NHibernate with conditions
                return Query_NHibernate
                    .Select(Projections.RootEntity())
                    //Cacheable() allows retrieval
                    .Cacheable()
                    //SingleOrDefault() allows at most one result
                    .SingleOrDefault();
            }
        }

        static void Main(string[] args)
        {
            //if command line argument is empty:
            if (args.Length == 0)
            {
                Console.WriteLine("Please enter a path to the input data XML file at the command line, as illustrated below:");
                Console.WriteLine("\nWeatherPrep.exe  C:\\iTree\\WeatherPrep\\input_output");
                //exit program when args.Length is zero
                System.Environment.Exit(1);
            }
            //define var for directory path, read input path from command line argument
            var dirPath = args[0];

            //define string for input data XML file path, adding @"" to front of directory path
            string filePath = @"" + dirPath + "\\WeatherPrepConfig.xml";

            //LAIDatabase is written to default dirPath; string for output LAIDatabase csv file path, adding @"" to front of directory path
            string LAIDatabase = @"" + dirPath + "\\LAI.csv";

            //WeatherDatabase is written to default dirPath; string for output WeatherDatabase csv file path, adding @"" to front of directory path
            string WeatherDatabase = @"" + dirPath + "\\Meteo.csv";

            //write file path
            Console.WriteLine("Below are the input data from: " + filePath + "\n");

            //If needed, replace all the "\" with "/", in the case if Visual Studio does not accept "\" in Path
            //filePath = filePath.Replace(@"\", "/");

            //Use XmlReader to open the .xml file
            using (XmlReader InputDataXml = XmlReader.Create(filePath))
            {
                // Read .xml file each line
                while (InputDataXml.Read())
                {
                    //Read from start
                    if (InputDataXml.IsStartElement())
                    {
                        //return only when you have a START tag 
                        // Check for each input case we expect a possible value for
                        switch (InputDataXml.Name.ToString())
                        {
                            //LookupLocationData function is currently setup to use place names and not LocationID
                            case "Model":
                                // Find Model 
                                Model = InputDataXml.ReadString();
                                Console.WriteLine("Model is: " + Model);
                                break;
                            case "Nation":
                                // Find Nation
                                Nation = InputDataXml.ReadString();
                                Console.WriteLine("Nation is: " + Nation);
                                break;
                            case "State":
                                // Find State
                                State = InputDataXml.ReadString();
                                Console.WriteLine("State is: " + State);
                                break;
                            case "County":
                                // Find County
                                County = InputDataXml.ReadString();
                                Console.WriteLine("County is: " + County);
                                break;
                            case "Place":
                                // Find Place
                                Place = InputDataXml.ReadString();
                                Console.WriteLine("Place is: " + Place);
                                break;
                            case "MaximumLAI":
                                // Find MaximumLAI
                                MaximumLAI = InputDataXml.ReadString();
                                Console.WriteLine("MaximumLAI is: " + MaximumLAI);
                                break;
                            case "EvergreenPercent":
                                // Find EvergreenPercent
                                EvergreenPercent = InputDataXml.ReadString();
                                Console.WriteLine("EvergreenPercent is: " + EvergreenPercent);
                                break;
                            //VegetationType used in CalcPrecipInterceptByCanopy function to determine canopy evaporation ammount
                            case "VegetationType":
                                //Find VegetationType and assign to vegType variable
                                VegetationType = InputDataXml.ReadString();
                                if (VegetationType.Equals("Tree"))
                                {
                                    vegType = VEGTYPE.TREE;
                                }
                                else if (VegetationType.Equals("Shrub"))
                                {
                                    vegType = VEGTYPE.SHRUB;
                                }
                                else if (VegetationType.Equals("Grass"))
                                {
                                    vegType = VEGTYPE.GRASS;
                                }
                                else
                                {
                                    // Error handle lack of vegetation type
                                    Console.WriteLine("Error: Invalid vegetation type. Should be Tree, Shrub, or Grass.");
                                }
                                Console.WriteLine("VegetationType is: " + VegetationType);
                                break;
                            case "Height_Tree_m":
                                // Find Height_Tree_m as double
                                Height_Tree_m = Int32.Parse(InputDataXml.ReadString());
                                Console.WriteLine("Height_Tree_m is: " + Height_Tree_m);
                                break;
                            case "Height_WindSensor_m":
                                // Find Height_WindSensor_m as double
                                Height_WindSensor_m = Int32.Parse(InputDataXml.ReadString());
                                Console.WriteLine("Height_WindSensor_m is: " + Height_WindSensor_m);
                                break;
                            case "StartYear":
                                // Find StartYear as integer
                                StartYear = Int32.Parse(InputDataXml.ReadString());
                                Console.WriteLine("StartYear is: " + StartYear);
                                break;
                            case "EndYear":
                                // Find EndYear as integer
                                EndYear = Int32.Parse(InputDataXml.ReadString());
                                Console.WriteLine("EndYear is: " + EndYear);
                                break;
                            case "SurfaceWeatherDataFile":
                                //Find SurfaceWeatherDataFile Path - this contains input weather data
                                SurfaceWeatherDataFile = InputDataXml.ReadString();
                                Console.WriteLine("Weather station data provided: " + SurfaceWeatherDataFile);
                                break;
                            case "PrecipitationDataCsv":
                                // Find PrecipitationDataCsv Path - optional input for replacing rainfall data
                                PrecipitationDataCsv = InputDataXml.ReadString();
                                //If PrecipitationDataCsv was provided then write it to the Console
                                if (PrecipitationDataCsv.Length > 0)
                                {
                                    Console.WriteLine("Precipitation data provided: " + PrecipitationDataCsv);
                                }
                                break;
                        }
                    }
                }

                //Handle empty data fields
                if (Height_WindSensor_m == 0)
                {
                    //Height_WindSensor_m = 10 is the default height used for wind measurement at airports
                    //Note: https://www.icams-portal.gov/resources/ofcm/fmh/siting/fcm-s4-2019.pdf
                    Height_WindSensor_m = 10;
                    Console.WriteLine("Add Height_WindSensor_m element to WeatherPrepConfig.xml. Height_WindSensor_m = " + Height_WindSensor_m);
                }
                if (Height_Tree_m == 0)
                {
                    //Height_Tree_m= 12 is a default height used tree height in urban areas
                    Height_Tree_m = 12;
                    Console.WriteLine("Add Height_Tree_m element to WeatherPrepConfig.xml. Height_Tree_m = " + Height_Tree_m);
                }

                //If needed, make EndYear same as StartYear    
                if (EndYear < StartYear) { EndYear = StartYear; }

                //If needed, replace all the "\" with "/", in the case if Visual Studio does not accept "\" in Path
                //SurfaceWeatherDataFile = SurfaceWeatherDataFile.Replace(@"\","/"); 
                //WeatherDatabase = WeatherDatabase.Replace(@"\", "/");
                //LAIDatabase = LAIDatabase.Replace(@"\", "/");

                // Open NHibernate session for functions using data from SQLite LocationSpecies DB
                using (ISession NHibernate_Session = SessionHelper.OpenSession())
                {
                    //Call Location LookupLocation function to use LocationSpecies data read from WeatherPrepConfig.xml and then used in the NHibernate Session
                    //Note: NHibernate_Session is NHibernate session
                    Location LocationSpecies_Location = LookupLocation(NHibernate_Session);

                    //Call CreateOutput class constructor, sending variables needed in next CreateOutput.cs 
                    CreateOutput LocationData_WeatherPrepConfig = new CreateOutput(double.Parse(MaximumLAI), double.Parse(EvergreenPercent), LAIDatabase,
                        StartYear, EndYear, vegType, SurfaceWeatherDataFile, PrecipitationDataCsv, WeatherDatabase, Model,
                        Height_WindSensor_m, Height_Tree_m);

                    //Call CreateOutputFiles, passing it the data from the WeatherPrepConfig file along with the paths to the NHibernate_Session and LocationSpecies_Location 
                    LocationData_WeatherPrepConfig.CreateOutputFiles(NHibernate_Session, LocationSpecies_Location);
                }
            }
        }
    }
}
